
{*****************************************************************************}
{
  Application   :-  Financial Calculator (Fin_Calc.exe) is a business utility
                    program.

  Version       :-  3.00

  Compiler      :-  Delphi versions 3, 4, 5, 6 or 7.

  OS            :-  Win9x/ME/2K/XP

  Author        :-  Leo D. Shih  <ldshih@ecn.ab.ca>

  Copyright     :-  (C)1999/2003 by Leo D. Shih, All rights reserved

  Latest Update :-  February 2003
}
{=============================================================================}
{
  Brief Description
  -----------------
  Fin_Calc.exe is a Financial Calculator, that calculates various types of
  time-dependent financial transactions, related to :
  1.  Mortgage (Regular payment & Detailed payment schedule);
  2.  Loan (Payment, Principal  & Term of Loans);
  3.  Investment  (Final Value, Initial Principal, Interest Rate & Investment
      term of Term Deposits);
  4.  Annuity with Withdrawals (Principal, Withdrawal amount, & number of
      withdrawals);
  5.  Annuity with Deposits (Final Value, Regular deposits & Annuity term).

  Context sensitive Help File (Fin_Calc.hlp) is included.
}
{==============================================================================}
{
  LEGAL INFO
  ----------

      Financial Calculator (Fin_Calc.exe)
      Copyright (C)1999/2003 by Leo D. Shih, All rights reserved

  This software (Fin_Calc.exe and associated source-code files) is
  currently distributed as Freeware, and is provided 'as-is', without
  warranty of any kind, either expressed or implied.  In no event
  shall the author (i.e. copyright holder) be liable for any losses or
  damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including redistribution and modification, provided that the following
  conditions are met.

  1.  This notice may not be removed or altered from any redistributions
      of this software.

  2.  The origin of this software must not be misrepresented; you must not
      claim that you wrote the original software.

  3.  Modified source code must be plainly marked as such, and must not be
      misrepresented as being the original software.

  4.  If you use this software, or any part thereof, in your application,
      appropriate credit to the author should be included in the "About-Form"
      and/or the documentation of your application.

  5.  If your application is to be distributed as commercial products,
      in addition to the above conditions, please write to the author
      for permission.
}
{=============================================================================}
{ Special Thanks
  ==============
      I would like to sincerely thank Martin Stoeckli for hos StoHtmlHelp.pas
      (Delphi's HtmlHelp API interface Unit)
}
{=============================================================================}


{$Include LsComp.inc}

unit Fc_unit1;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, Printers,
  LsNumericEdit, ComCtrls, ToolWin
{$IFDEF D4_OR_HIGHER},  //2.21
  ImgList
{$ENDIF}
{$IFDEF D6_OR_HIGHER},  //3.0hh
  StoHtmlHelp
{$ENDIF};



type
  TFc_Form1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Print1: TMenuItem;
    PrintSetup1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Investment1: TMenuItem;
    FinalValue1: TMenuItem;
    InitialPrincipal1: TMenuItem;
    InterestRate1: TMenuItem;
    Investmentterm1: TMenuItem;
    Loans1: TMenuItem;
    Payment1: TMenuItem;
    Principal1: TMenuItem;
    LoanTerm1: TMenuItem;
    Annuity1: TMenuItem;
    FinalValue2: TMenuItem;
    RegularDeposit1: TMenuItem;
    AnnuityTerm1: TMenuItem;
    Mortgage1: TMenuItem;
    Panel1: TPanel;
    Panel2: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Help1: TMenuItem;
    About1: TMenuItem;
    Fc_Memo1: TMemo;
    ClearBtn2: TBitBtn;
    PrintDialog1: TPrintDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    Contents1: TMenuItem;
    AnnuitywithDeposit1: TMenuItem;
    N2: TMenuItem;
    AnnuitywithWithdrawals1: TMenuItem;
    Principal2: TMenuItem;
    WithdrawalAmount1: TMenuItem;
    NumberofWithdrawals1: TMenuItem;
    Panel3: TPanel;
    Label6: TLabel;
    Panel4: TPanel;
    PrintBtn: TBitBtn;
    Panel5: TPanel;
    Panel6: TPanel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Image1: TImage;
    Panel7: TPanel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    ToolBar1: TToolBar;
    CalcBtn: TToolButton;
    ImageList1: TImageList;
    RptBtn: TToolButton;
    CancelBtn: TToolButton;
    HelpBtn1: TToolButton;
    ImageList2: TImageList;
    ExitBtn: TToolButton;
    Label7: TLabel;

    procedure CalcBtnClick(Sender: TObject);
    procedure RptBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FinalValue1Click(Sender: TObject);
    procedure InitialPrincipal1Click(Sender: TObject);
    procedure InterestRate1Click(Sender: TObject);
    procedure Investmentterm1Click(Sender: TObject);
    procedure Payment1Click(Sender: TObject);
    procedure Principal1Click(Sender: TObject);
    procedure LoanTerm1Click(Sender: TObject);
    procedure FinalValue2Click(Sender: TObject);
    procedure RegularDeposit1Click(Sender: TObject);
    procedure AnnuityTerm1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Principal2Click(Sender: TObject);
    procedure WithdrawalAmount1Click(Sender: TObject);
    procedure NumberofWithdrawals1Click(Sender: TObject);
    procedure ClearBtn2Click(Sender: TObject);
    procedure PrintSetup1Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure Contents1Click(Sender: TObject);
    procedure HelpBtn1Click(Sender: TObject);
    procedure ExitBtnClick(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure PrintBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EnableEdits;
    procedure DisableEdits;
    procedure Mortgage1Click(Sender: TObject);
    procedure ProcessMortgage;
    procedure ResetEdit629;  //3.0
    procedure ResetEdit125;  //3.0
    procedure SetEditMasks;
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);  //3.0hh

  private
    { Private declarations }
    Compute: Boolean;
    FinFunc: Integer;
    function FinalValue: Double;
    function InitPrincipal: Double;
    function AnnualInt: Double;
    function InvestTerm: integer;
    function LPayment: Double;
    function LPrincipal: Double;
    function LTerm: integer;
    function AFValue: Double;
    function ADeposit: Double;
    function ATerm: integer;
    function WDrawPrin: Double;
    function WDrawAmt: Double;
    function WDTerm: integer;

    procedure ShowComponents;
    procedure ClearComponents;
    procedure ClearEdits;

  public
    { Public declarations }
    NewMortgData: Boolean;  //2.0
    InValues: Boolean;      //2.0
    GetOut: Boolean;        //2.0

    Edit1: TLsNumericEdit;  //3.0
    Edit2: TLsNumericEdit;  //3.0
    Edit3: TLsNumericEdit;  //3.0
    Edit4: TLsNumericEdit;  //3.0
    Edit5: TLsNumericEdit;  //3.0
    Edit6: TLsNumericEdit;  //3.0
    Edit7: TLsNumericEdit;  //3.0
    Edit8: TLsNumericEdit;  //3.0
    Edit9: TLsNumericEdit;  //3.0
  end;

  {Global Financil Functions}

  function InvestFV(Principal, InterestRate: Double;
    CompoundYear, NumberMonths: Integer): Double;
  function InvestP(FinalValue, InterestRate: Double;
    CompoundYear, NumberMonths: integer): Double;
  function InvestIR(Principal, FinalValue: Double;
    CompoundYear, NumberMonths: integer): Double;
  function InvestNM(Principal, FinalValue, InterestRate: Double;
    CompoundYear: integer): integer;
  function LoanPay(Principal, InterestRate: Double; TotalNumPay,
    NumPayPerYr: integer): Double;
  function LoanPrin(Payment, InterestRate: Double; TotalNumPay,
    NumPayPerYr: integer): Double;
  function LoanTNP(Principal, Payment, InterestRate: Double;
    NumPayPerYr: integer): integer;
  function AnnuFV(Deposit, InterestRate: Double;
    TotalNumDep, NumDepPerYr: integer): Double;
  function AnnuDep(FinalValue, InterestRate: Double;
    TotalNumDep, NumDepPerYr: integer): Double;
  function AnnuTND(Deposit, FinalValue, InterestRate: Double;
    NumDepPerYr: integer): integer;
  function WDrawP(Withdrawal, InterestRate: Double;
    TotalNumW, NumWPerYr: integer): Double;
  function WDrawWD(Principal, InterestRate: Double;
    TotalNumW, NumWPerYr: integer): Double;
  function WDrawTNW(Principal, Withdrawal, InterestRate: Double;
    NumWPerYr: byte): integer;

var
  Fc_Form1: TFc_Form1;


implementation

{$R *.DFM}
//{$R WinXP.res}

uses Fc_Unit2;

const
  HH_HELP_CONTEXT = $F;

{ ======  Global Financil Function ====== }

function InvestFV(Principal, InterestRate: Double;
  CompoundYear, NumberMonths: Integer): Double;
var
  Growth: Double;
begin
  Growth := exp(NumberMonths / 12.0 * CompoundYear *
    ln(1.0 + InterestRate / 100.0 / CompoundYear));
  Result := (Principal * Growth);
end;


function InvestP(FinalValue, InterestRate: Double;
  CompoundYear, NumberMonths: integer): Double;
var
  Growth: Double;
begin
  Growth := exp(NumberMonths / 12.0 * CompoundYear *
    ln(1.0 + InterestRate / 100.0 / CompoundYear));
  Result := (FinalValue / Growth);
end;


function InvestIR(Principal, FinalValue: Double;
  CompoundYear, NumberMonths: integer): Double;
begin
  Result := CompoundYear * (exp(ln(FinalValue / Principal) /
    CompoundYear / NumberMonths * 12.0) - 1.0) * 100.0
end;


function InvestNM(Principal, FinalValue, InterestRate: Double;
  CompoundYear: integer): integer;
var
  RealNM: Double;
  NumMonth: integer;
begin
  RealNM := ln(FinalValue / Principal) /
    ln(InterestRate / 100.0 / CompoundYear + 1.0) /
    CompoundYear * 12;
  if frac(RealNM) = 0.0 then
    NumMonth := trunc(RealNM)
  else
    NumMonth := trunc(RealNM) + 1;
  result := NumMonth;
end;


function LoanPay(Principal, InterestRate: Double; TotalNumPay,
  NumPayPerYr: integer): Double;
var
  Temp: Double;
begin
  Temp := exp(-TotalNumPay * ln(1.0 + interestRate / 100.0 / NumPayPerYr));
  Result := Principal * InterestRate / 100.0 / NumPayPerYr / (1.0 - Temp);
end;


function LoanPrin(Payment, InterestRate: Double; TotalNumPay,
  NumPayPerYr: integer): Double;
var
  Temp: Double;
begin
  Temp := exp(-TotalNumPay * ln(1.0 + InterestRate / 100.0 / NumPayPerYr));
  Result := Payment * NumPayPerYr * 100.0 / InterestRate * (1.0 - Temp);
end;


function LoanTNP(Principal, Payment, InterestRate: Double;
  NumPayPerYr: integer): integer;
var
  Temp, RealTNP: Double;
  LoanTerm: integer;
begin
  Temp := Principal * InterestRate / 100.0 / Payment / NumPayPerYr;
  if Temp > 1 then
    RealTNP := 0.0
  else
    RealTNP := -ln(1.0 - Temp) / ln(1.0 + InterestRate / 100.0 / NumPayPerYr);
  if frac(RealTNP) = 0.0 then
    LoanTerm := trunc(RealTNP)
  else
    LoanTerm := trunc(RealTNP) + 1;
  Result := LoanTerm;
end;


function AnnuFV(Deposit, InterestRate: Double;
  TotalNumDep, NumDepPerYr: integer): Double;
var
  Product: Double;
  j: integer;
begin
  Product := 1.0;
  for j := 1 to TotalNumDep do
    Product := Product * (1.0 + InterestRate / 100.0 / NumDepPerYr);
  Result := Deposit * (Product - 1.0) * NumDepPerYr * 100.0 / InterestRate;
end;


function AnnuDep(FinalValue, InterestRate: Double;
  TotalNumDep, NumDepPerYr: integer): Double;
var
  Product: Double;
  j: integer;
begin
  Product := 1.0;
  for j := 1 to TotalNumDep do
    Product := Product * (1.0 + InterestRate / 100.0 / NumDepPerYr);
  Result := FinalValue * InterestRate / 100.0 / NumDepPerYr / (Product - 1.0);
end;


function AnnuTND(Deposit, FinalValue, InterestRate: Double;
  NumDepPerYr: integer): integer;
var
  Temp, RealTND: Double;
  TNumDep: integer;
begin
  Temp := InterestRate / 100.0 / NumDepPerYr;
  RealTND := ln(1.0 + FinalValue * Temp / Deposit) / ln(1.0 + Temp);
  if frac(RealTND) = 0.0 then
    TNumDep := trunc(RealTND)
  else
    TNumDep := trunc(RealTND) + 1;
  Result := TNumDep;
end;


function WDrawP(Withdrawal, InterestRate: Double;
  TotalNumW, NumWPerYr: integer): Double;
var
  Product: Double;
  j: integer;
begin
  Product := 1.0;
  for j := 1 to TotalNumW do
    Product := Product * (1.0 + InterestRate / 100.0 / NumWPerYr);
  Result := Withdrawal * NumWPerYr * (1.0 - 1.0 / Product) *
    100.0 / InterestRate;
end;


function WDrawWD(Principal, InterestRate: Double;
  TotalNumW, NumWPerYr: integer): Double;
var
  Product: Double;
  j: integer;
begin
  Product := 1.0;
  for j := 1 to TotalNumW do
    Product := Product * (1.0 + InterestRate / 100.0 / NumWPerYr);
  Result := Principal * InterestRate / 100.0 / NumWPerYr *
    (1.0 + 1.0 / (Product - 1.0));
end;


function WDrawTNW(Principal, Withdrawal, InterestRate: double;
  NumWPerYr: byte): integer;
var
  Temp, RealTNW: Double;
  TNWDraw: integer;
begin
  Temp := 1.0 - Principal * InterestRate / 100.0 / NumWPerYr / Withdrawal;
  if Temp <= 0.0 then
    RealTNW := 0
  else
    RealTNW := -ln(Temp) / ln(1.0 + InterestRate / 100.0 / NumWPerYr);
  if frac(RealTNW) = 0.0 then
    TNWDraw := trunc(RealTNW)
  else
    TNWDraw := trunc(RealTNW) + 1;
  result := TNWDraw;
end;

// end of Global financial Subroutines

{ ======= General Routines ======= }

procedure TFc_Form1.FormCreate(Sender: TObject);  //2.0 >>
begin
  Edit1 := TLsNumericEdit.Create(Self);  //3.0
  with Edit1 do
  begin
    Parent := Panel1;
    Left := 308;
    Top := 20;
    Height := 23;
    Width := 121;
    Font.Name := 'MS Sans Serif';
    Font.Size := 9;
    Font.Color := clBlack;
    TabStop := True;
    TabOrder := 0;
    OnKeyDown := EditKeyDown;   //3.0
  end;
  Edit2 := TLsNumericEdit.Create(Self);  //3.0
  with Edit2 do
  begin
    Parent := Panel1;
    Left := 308;
    Top := 48;
    Height := 23;
    Width := 121;
    Font.Name := 'MS Sans Serif';
    Font.Size := 9;
    Font.Color := clBlack;
    TabStop := True;
    TabOrder := 1;
    OnKeyDown := EditKeyDown;   //3.0
  end;
  Edit3 := TLsNumericEdit.Create(Self);  //3.0
  with Edit3 do
  begin
    Parent := Panel1;
    Left := 308;
    Top := 76;
    Height := 23;
    Width := 121;
    Font.Name := 'MS Sans Serif';
    Font.Size := 9;
    Font.Color := clBlack;
    TabStop := True;
    TabOrder := 2;
    OnKeyDown := EditKeyDown;   //3.0
  end;
  Edit4 := TLsNumericEdit.Create(Self);  //3.0
  with Edit4 do
  begin
    Parent := Panel1;
    Left := 308;
    Top := 104;
    Height := 23;
    Width := 121;
    Font.Name := 'MS Sans Serif';
    Font.Size := 9;
    Font.Color := clBlack;
    TabStop := True;
    TabOrder := 3;
    OnKeyDown := EditKeyDown;  //3.0
  end;
  Edit5 := TLsNumericEdit.Create(Self);  //3.0
  with Edit5 do
  begin
    Parent := Panel3;
    Left := 308;
    Top := 12; //8;
    Height := 29;
    Width := 121;
    NormalColor := clBlack;
    FocusedColor := clBlack;
    Font.Name := 'MS Sans Serif';
    Font.Size := 10; //3.0
    Font.Style := [fsBold];
    Font.Color := clLime;
    HideSelection := True;
    TabStop := False;
    OnKeyDown := EditKeyDown;   //3.0
  end;
  Edit6 := TLsNumericEdit.Create(Self);  //3.0
  with Edit6 do
  begin
    Parent := Panel7;
    Left := 272;
    Top := 16;
    Height := 23;
    Width := 97;
    Font.Name := 'MS Sans Serif';
    Font.Size := 9;
    Font.Color := clBlack;
    TabStop := True;
    OnKeyDown := EditKeyDown;   //3.0
  end;
  Edit7 := TLsNumericEdit.Create(Self);    //3.0
  with Edit7 do
  begin
    Parent := Panel7;
    Left := 272;
    Top := 44;
    Height := 23;
    Width := 97;
    Mask := '#0.00000';
    Font.Name := 'MS Sans Serif';
    Font.Size := 9;
    Font.Color := clBlack;
    TabStop := True;
    OnKeyDown := EditKeyDown;   //3.0
  end;
  Edit8 := TLsNumericEdit.Create(Self);     //3.0
  with Edit8 do
  begin
    Parent := Panel7;
    Left := 272;
    Top := 72;
    Height := 23;
    Width := 97;
    Mask := '#0';
    Font.Name := 'MS Sans Serif';
    Font.Size := 9;
    Font.Color := clBlack;
    TabStop := True;
    OnKeyDown := EditKeyDown;   //3.0
  end;
  Edit9 := TLsNumericEdit.Create(Self);    //3.0
  with Edit9 do
  begin
    Parent := Panel7;
    Left := 272;
    Top := 100;
    Height := 23;
    Width := 97;
    Mask := '#';
    Font.Name := 'MS Sans Serif';
    Font.Size := 9;
    Font.Color := clBlack;
    TabStop := True;
    OnKeyDown := EditKeyDown;   //3.0
  end;
{$IFDEF D6_OR_HIGHER}  //3.0hh>
  Application.HelpFile := 'FIN_CALC.CHM';
{$ELSE}
  Application.HelpFile := 'FIN_CALC.HLP';
{$ENDIF}   //3.0hh<
  DisableEdits;
  Compute := False;  //2.1
  Panel5.visible := True;
end;  //FormCreate  //2.0 <<


procedure TFc_Form1.FormActivate(Sender: TObject);
begin
  NewMortgData := True;
  Panel5.visible := True;
  Panel2.Caption := 'Select financial transactions from the Main Menu';
end;  //FormActivate


procedure TFc_Form1.ExitBtnClick(Sender: TObject);
begin
  Close;
end;  //ExitBtnClick


procedure TFc_Form1.ShowComponents;
begin
    Panel1.Visible := True;
    Panel5.Visible := False;
    Fc_Memo1.Visible := False;
    Label1.visible := True;
    CalcBtn.Enabled := True;
    RptBtn.Enabled := True;
    CancelBtn.Enabled := True;  //2.1
    HelpBtn1.Enabled := True;
    CancelBtn.Visible := True;
    ClearBtn2.visible := False;
    PrintBtn.visible := False;
    Print1.Enabled := False;
    PrintSetup1.Enabled := False;
    Panel2.Caption := '';
    EnableEdits;
end;  //ShowComponents


procedure TFc_Form1.EnableEdits;
begin
  Edit1.Visible := True;  //2.0
  Edit2.Visible := True;  //2.0
  Edit3.Visible := True;  //2.0
  Edit4.Visible := True;  //2.0
  Edit5.Visible := True;  //2.0
end;  //EnableEdits


procedure TFc_Form1.DisableEdits;
begin
  Edit1.Visible := False;  //2.0
  Edit2.Visible := False;  //2.0
  Edit3.Visible := False;  //2.0
  Edit4.Visible := False;  //2.0
  Edit5.Visible := False;  //2.0
end;  //DisableEdits


procedure TFc_Form1.ResetEdit629;  //3.0>
begin
  Edit6.Value := 0;
  Edit7.Value := 0;
  Edit8.Value := 0;
  Edit9.Value := 0;
  Edit6.Text := '';
  Edit7.Text := '';
  Edit8.Text := '';
  Edit9.Text := '';
end;  //ResetEdit629  //3.0<


procedure TFc_Form1.ResetEdit125;  //3.0>
begin
  Edit1.Value := 0;
  Edit2.Value := 0;
  Edit3.Value := 0;
  Edit4.Value := 0;
  Edit5.Value := 0;
  Edit1.Text := '';
  Edit2.Text := '';
  Edit3.Text := '';
  Edit4.Text := '';
  Edit5.Text := '';
end;  //ResetEdit125  //3.0<


procedure TFc_Form1.ClearComponents;
begin
    Label1.Caption := '';
    Label2.Caption := '';
    Label3.Caption := '';
    Label4.Caption := '';
    Label5.Caption := '';
    Label6.Caption := '';

    ResetEdit125;  //3.0
    Panel5.Visible := True;
    Label1.visible := False;
    CalcBtn.Enabled := False;
    RptBtn.Enabled := False;
    CancelBtn.Enabled := False;  //2.1
end;  //ClearComponents


procedure TFc_Form1.ClearBtn2Click(Sender: TObject);
begin
  Fc_Memo1.Clear;
  Fc_Form1.Caption := 'Financial Calculator';
  Fc_Memo1.visible := False;
  Image1.Visible := True;
  Label1.visible := False;
  Label6.Caption := '';
  Edit5.Text := '';
  Panel6.visible := False;
  ClearBtn2.visible := False;
  PrintBtn.Visible := False;
  Print1.Enabled := False;
  PrintSetup1.Enabled := False;
  ResetEdit629;  //3.0
  Panel7.Visible := False;
  Panel1.Visible := True;
  Panel5.Visible := True;
  File1.Enabled := True;   //2.2>>
  Investment1.Enabled := True;
  Loans1.Enabled := True;
  Annuity1.Enabled := True;  //2.2<<
  CalcBtn.Enabled := False;  //2.21>>
  RptBtn.Enabled := False;
  CancelBtn.Enabled := False;  //2.21<<
  FinFunc := 0;
end;  //ClearBtn2Click


procedure TFc_Form1.ClearEdits;
begin
  if FinFunc in [1..13] then
    ResetEdit125
  else if FinFunc = 14 then
    ResetEdit629;
end;  //ClearEdits


procedure TFc_Form1.SetEditMasks;  //3.0>
begin
  Case FinFunc of
    1,2,5,6,8,9,11,12:
      begin
        Edit1.Mask := ',##0.00';
        Edit2.Mask := '0.00000';
        Edit3.Mask := '0';
        Edit4.Mask := '0';
        Edit5.Mask := ',##0.00';
      end;
    3:
      begin
        Edit1.Mask := ',##0.00';
        Edit2.Mask := ',##0.00';
        Edit3.Mask := '0';
        Edit4.Mask := '0';
        EDit5.Mask := '0.00000';
      end;
    4,7,10,13:
      begin
        Edit1.Mask := ',##0.00';
        Edit2.Mask := ',##0.00';
        Edit3.Mask := '0.00000';
        Edit4.Mask := '0';
        Edit5.Mask := '0';
      end;
  end;
  ClearEdits;
end;  //setEditMasks   //3.0<


procedure TFc_Form1.CalcBtnClick(Sender: TObject);
begin
  if (FinFunc in [1..13]) then  //2.1
  begin
    if (Edit1.Text = '') or (Edit2.Text = '') or
      (Edit3.Text = '') or (Edit4.Text = '') then
    begin
      MessageDlg('Data entries incomplete!', mtError, [mbOK],0);
      exit;
    end;
    case FinFunc of
      1: FinalValue;
      2: InitPrincipal;
      3: AnnualInt;
      4: InvestTerm;
      5: LPayment;
      6: LPrincipal;
      7: LTerm;
      8: AFValue;
      9: ADeposit;
      10: ATerm;
      11: WDrawPrin;
      12: WDrawAmt;
      13: WDTerm;
    end;
  end
  else if (FinFunc = 14) then   //2.1 >>
  begin
    InValues := False;
    NewMortgData := False;
    if (Edit6.Text = '0.00') or (Edit6.Value <= 0) then  //3.0
    begin
      MessageBeep(0);  //3.0
      MessageDlg('Invalid Principal Value', mtInformation, [mbOK], 0);
      Mortgage1Click(Sender);
      Edit6.SetFocus;
    end
    else if (Edit7.Value <= 0) or  (Edit7.Value > 100) then //3.0
    begin
      MessageBeep(0);  //3.0
      MessageDlg('Invalid Interest Rate', mtError, [mbOK], 0);
      Mortgage1Click(Sender);
      Edit7.SetFocus;
    end
    else if (Edit8.Text = '0') or  (Edit8.ValueAsInt <= 0) then  //3.0
    begin
      MessageBeep(0);  //3.0
      MessageDlg('Invalid Term of Loan', mtError, [mbOK], 0);
      Mortgage1Click(Sender);
      Edit8.SetFocus;
    end
    else if (Edit9.Text = '0') or not (Edit9.ValueAsInt in [1..2]) then //3.0
    begin
      MessageBeep(0);  //3.0
      MessageDlg('Number of payment per month should be 1 or 2', mtError, [mbOK], 0);
      Mortgage1Click(Sender);
      Edit9.SetFocus;
    end
    else begin
      GetOut := False;
      InValues := True;
      NewMortgData := True;
      Panel6.Visible := True;
      ProcessMortgage;
    end;
  end;  //else if FinFunc=14  //2.1 <<
end;


procedure TFc_Form1.RptBtnClick(Sender: TObject);
begin
  ClearEdits;
//  Fc_Form1.Refresh;         //3.0>
  if FinFunc in [1..13] then
    Edit1.SetFocus           //3.0<
  else if FinFunc = 14 then   //2.1
    Edit6.SetFocus;
end;


procedure TFc_Form1.CancelBtnClick(Sender: TObject);
begin
  Image1.Visible := True;  //3.0
  if FinFunc in [1..13] then  //2.1
  begin
    Compute := False;
    ClearComponents;
    DisableEdits;
  end
  else if FinFunc = 14 then
  begin
    if NewMortgData = True then
      ResetEdit629;  //3.0
    ClearComponents;
    Fc_Memo1.Clear;
    Fc_Memo1.Visible := False;
    ClearBtn2.Visible := False;
    PrintBtn.Visible := False;  //2.21
    Print1.Enabled := False;
    PrintSetup1.Enabled := False;
    Panel1.Visible := True;
    Panel5.Visible := True;
    Panel6.Visible := False;
    Panel7.Visible := False;
    InValues := False;
    GetOut := True;
  end;  //2.1 <<
  File1.Enabled := True;  //2.2>>
  Investment1.Enabled := True;
  Loans1.Enabled := True;
  Annuity1.Enabled := True;  //2.2<<
  FinFunc := 0;  //2.1
  Caption := 'Financial Calculator';
end;  //CancelBtnClick


procedure TFc_Form1.EditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_F1) then
  begin
    HelpBtn1Click(Self);  //3.0hh
    Key := 0;
  end
  else if (key = VK_Return)  then
    if (ActiveControl = Edit4) or (ActiveControl = Edit9) then
    begin
      Application.ProcessMessages;
//      PostMessage(Handle, WM_KEYDOWN, VK_TAB, 0);   //2.1
      CalcBtnClick(self);
    end;
end;  //EditKeyDown


procedure TFc_Form1.Exit1Click(Sender: TObject);
begin
  Close;
end;  //Exit1Click

procedure TFc_Form1.About1Click(Sender: TObject);
begin
  AboutBox.ShowModal;
end;  //About1Click


procedure TFc_Form1.PrintSetup1Click(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
end;  //PrintSetup1Click


procedure TFc_Form1.Print1Click(Sender: TObject);
begin
  PrintBtnClick(Sender);
end;  //Print1Click


procedure TFc_Form1.FinalValue1Click(Sender: TObject);
begin
  FinFunc := 1;
  ClearEdits;
  SetEditMasks;   //3.0
  Fc_Form1.Caption := 'Financial Calculator (INVESTMENT - Final Value)';
  Label1.Caption := 'Calculate the final value of an investment, with given investment specifications';
  Label2.Caption := 'Principal (Initial amount invested)';
  Label3.Caption := 'Annual Interest Rate (in percent)';
  Label4.Caption := 'Number of compounding periods per year';
  Label5.Caption := 'Term - Length of investment (in months)';
  Label6.Caption := 'Final Value at maturity';
  ShowComponents;
  Edit1.SetFocus;
end;  //FinalValue1Click


procedure TFc_Form1.InitialPrincipal1Click(Sender: TObject);
begin
  FinFunc := 2;
  ClearEdits;
  SetEditMasks;   //3.0
  Fc_Form1.Caption := 'Financial Calculator (INVESTMENT - Principal)';
  Label1.Caption := 'Calculate the initial investment required to reach a certain final value';
  Label2.Caption := 'Final Value of the investment';
  Label3.Caption := 'Annual Interest Rate (in percent)';
  Label4.Caption := 'Number of compounding periods per year';
  Label5.Caption := 'Term - Length of investment (in months)';
  Label6.Caption := 'Principal (initial investment)';
  ShowComponents;
  Edit1.SetFocus;
end;   //InitialPrincipal1Click


procedure TFc_Form1.InterestRate1Click(Sender: TObject);
begin
  FinFunc := 3;
  ClearEdits;
  SetEditMasks;   //3.0
  Fc_Form1.Caption := 'Financial Calculator (INVESTMENT - Interest Rate)';
  Label1.Caption := 'Calculate the interest rate required to produce a specified return';
  Label2.Caption := 'Principal (initial amount invested)';
  Label3.Caption := 'Final Value of the investment';
  Label4.Caption := 'Number of compounding periods per year';
  Label5.Caption := 'Term - Length of investment (in months)';
  Label6.Caption := 'Annual Interest Rate (in percent)';
  ShowComponents;
  Edit1.SetFocus;
end;   //InterestRate1Click


procedure TFc_Form1.Investmentterm1Click(Sender: TObject);
begin
  FinFunc := 4;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (INVESTMENT - Term)';
  Label1.Caption := 'Calculate the Term required for an investment to mature';
  Label2.Caption := 'Principal (initial amount invested)';
  Label3.Caption := 'Final Value of the investment';
  Label4.Caption := 'Annual interest rate (in percent)';
  Label5.Caption := 'Number of compounding periods per year';
  Label6.Caption := 'Term - Length of investment (in months)';
  ShowComponents;
  Edit1.SetFocus;
end;  //Investmentterm1Click


procedure TFc_Form1.Payment1Click(Sender: TObject);
begin
  FinFunc := 5;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (LOAN - Payment)';
  Label1.Caption := 'Calculate the amount of each payment required to payback a loan';
  Label2.Caption := 'Principal (The amount borrowed)';
  Label3.Caption := 'Annual interest rate (in percent)';
  Label4.Caption := 'Term (Total number of payments)';
  Label5.Caption := 'Number of payments per year';
  Label6.Caption := 'Amount of each payment';
  ShowComponents;
  Edit1.SetFocus;
end;  //Payment1Click


procedure TFc_Form1.Principal1Click(Sender: TObject);
begin
  FinFunc := 6;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (LOAN - Principal)';
  Label1.Caption := 'Calculate the principal (initial amount borrowed) for a loan';
  Label2.Caption := 'Amount of each payment';
  Label3.Caption := 'Annual interest rate (in percent)';
  Label4.Caption := 'Term (Total number of payments)';
  Label5.Caption := 'Number of payments per year';
  Label6.Caption := 'Principal (initial amount borrowed)';
  ShowComponents;
  Edit1.SetFocus;
end;  //Principal1Click


procedure TFc_Form1.LoanTerm1Click(Sender: TObject);
begin
  FinFunc := 7;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (LOAN - Term)';
  Label1.Caption := 'Calculate the term (total number of payments) required to pay back a loan';
  Label2.Caption := 'Principal (the amount borrowed)';
  Label3.Caption := 'Amount of each payment';
  Label4.Caption := 'Annual interest rate (in percent)';
  Label5.Caption := 'Number of payments per year';
  Label6.Caption := 'Total number of payments';
  ShowComponents;
  Edit1.SetFocus;
end;  //LoanTerm1Click


procedure TFc_Form1.FinalValue2Click(Sender: TObject);
begin
  FinFunc := 8;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (ANNUITY - Final Value)';
  Label1.Caption := 'Calculate the final value of an annuity with deposits made in regular intervals';
  Label2.Caption := 'Amount of each deposit';
  Label3.Caption := 'Annual interest rate (in percent)';
  Label4.Caption := 'Term (Total number of deposits)';
  Label5.Caption := 'Number of deposits per year';
  Label6.Caption := 'Final value of the annuity';
  ShowComponents;
  Edit1.SetFocus;
end;  //FinalValue2Click


procedure TFc_Form1.RegularDeposit1Click(Sender: TObject);
begin
  FinFunc := 9;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (ANNUITY - Amount of Deposit)';
  Label1.Caption := 'Calculate the amount of each deposit required to reach a specified final value';
  Label2.Caption := 'Final value of the annuity';
  Label3.Caption := 'Annual interest rate (in percent)';
  Label4.Caption := 'Term (Total number of deposits)';
  Label5.Caption := 'Number of deposits per year';
  Label6.Caption := 'Amount of each deposit';
  ShowComponents;
  Edit1.SetFocus;
end;  //RegularDeposit1Click


procedure TFc_Form1.AnnuityTerm1Click(Sender: TObject);
begin
  FinFunc := 10;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (ANNUITY - Term)';
  Label1.Caption := 'Calculate the total number of deposits required for an annuity to mature';
  Label2.Caption := 'Amount of each deposit';
  Label3.Caption := 'Final value of the annuity';
  Label4.Caption := 'Annual interest rate (in percent)';
  Label5.Caption := 'Number of deposits made each year';
  Label6.Caption := 'Term (total number of deposits)';
  ShowComponents;
  Edit1.SetFocus;
end;  //AnnuityTerm1Click


procedure TFc_Form1.Principal2Click(Sender: TObject);
begin
  FinFunc := 11;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (ANNUITY - Principal)';
  Label1.Caption := 'Principal required to sustain an investment with regular withdrawals';
  Label2.Caption := 'Amount of each withdrawal';
  Label3.Caption := 'Annual interest rate (in percent)';
  Label4.Caption := 'Term (total number of withdrawals)';
  Label5.Caption := 'Number of withdrawals made each year';
  Label6.Caption := 'Principal (initial investment) required';
  ShowComponents;
  Edit1.SetFocus;
end;  //Principal2Click


procedure TFc_Form1.WithdrawalAmount1Click(Sender: TObject);
begin
  FinFunc := 12;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (ANNUITY - Withdrawal Amount)';
  Label1.Caption := 'Calculate the withdrawal amount, enabling an investment to sustain itself';
  Label2.Caption := 'Principal (initial investment)';
  Label3.Caption := 'Annual interest rate (in percent)';
  Label4.Caption := 'Term (total number of withdrawals)';
  Label5.Caption := 'Number of withdrawals made each year';
  Label6.Caption := 'Maximum amount of each withdrawal';
  ShowComponents;
  Edit1.SetFocus;
end;  //WithdrawalAmount1Click


procedure TFc_Form1.NumberofWithdrawals1Click(Sender: TObject);
begin
  FinFunc := 13;
  ClearEdits;
  SetEditMasks;  //3.0
  Fc_Form1.Caption := 'Financial Calculator (ANNUITY - Number of Withdrawals)';
  Label1.Caption := 'Calculate the number of withdrawals until an investment is depleted to zero';
  Label2.Caption := 'Principal (initial investment)';
  Label3.Caption := 'Amount of each withdrawal';
  Label4.Caption := 'Annual interest rate (in percent)';
  Label5.Caption := 'Number of withdrawals made each year';
  Label6.Caption := 'Total number of withdrawals';
  ShowComponents;
  Edit1.SetFocus;
end;  //NumberofWithdrawals1Click



{ ======= Financial Calculations routines ======= }

function TFc_Form1.FinalValue: Double;
var
  InitAmt, IntRate: Double;
  CompFreq, NumMon: Integer;
begin
  InitAmt := Edit1.Value;    //3.0<
  IntRate := Edit2.Value;
  CompFreq := Edit3.ValueAsInt;
  NumMon := Edit4.ValueAsInt;  //3.0<

  Result := InvestFV(InitAmt, IntRate, CompFreq, NumMon);
//  Edit5.Text := FormatFloat('#,##0.00', Result);  //3.0
  Edit5.Value := Result;                            //3.0
end;  //FinalValue


function TFc_Form1.InitPrincipal: Double;
var
  IntRate, FinalVal: Double;
  CompFreq, NumMon: Integer;
begin
  FinalVal := Edit1.Value;     //3.0>
  IntRate := Edit2.Value;
  CompFreq := Edit3.ValueAsInt;
  NumMon := Edit4.ValueAsInt;  //3.0<

  Result := InvestP(FinalVal, IntRate, CompFreq, NumMon);
//  Edit5.Text := FormatFloat('#,##0.00', Result);  //3.0
  Edit5.Value := Result;  //3.0
end;  //InitPrincipal


function TFc_Form1.AnnualInt: Double;
var
  InitAmt, FinalVal: Double;
  CompFreq, NumMon: Integer;
begin
  InitAmt := Edit1.Value;       //3.0>
  FinalVal := Edit2.Value;
  CompFreq := Edit3.ValueAsInt;
  NumMon := Edit4.ValueAsInt;   //3.0<

  Result := InvestIR(InitAmt, FinalVal, CompFreq, NumMon);
//  Edit5.Text := FormatFloat('0.000000', Result);
  Edit5.Value := Result;
end;  //AnnualInt


function TFc_Form1.InvestTerm: integer;
var
  InitAmt, IntRate, FinalVal: Double;
  CompFreq: Integer;
begin
  InitAmt := Edit1.Value;        //3.0>
  FinalVal := Edit2.Value;
  IntRate := Edit3.Value;
  CompFreq := Edit4.ValueAsInt;  //3.0<

  Result := InvestNM(InitAmt, FinalVal, IntRate, CompFreq);
//  Edit5.Text := Format('%10.0d', [Result]);  //3.0
  Edit5.ValueAsInt := Result;                  //3.0
end;  //InvestTerm


function TFc_Form1.LPayment: Double;
var
  PrinAmt, IntRate: Double;
  TNumPay, NPayPerYr: Integer;
begin
  PrinAmt := Edit1.Value;         //3.0>
  IntRate := Edit2.Value;
  TNumPay := Edit3.ValueAsInt;
  NPayPerYr := Edit4.ValueAsInt;  //3.0<

  Result := LoanPay(PrinAmt, IntRate, TNumPay, NPayPerYr);
//  Edit5.Text := FormatFloat('#,##0.00', Result);  //3.0
  Edit5.Value := Result;                            //3.0
end;  //LPayment


function TFc_Form1.LPrincipal: Double;
var
  IntRate, RPayment: Double;
  TNumPay, NPayPerYr: Integer;
begin
  RPayment := Edit1.Value;        //3.0>
  IntRate := Edit2.Value;
  TNumPay := Edit3.ValueAsInt;
  NPayPerYr := Edit4.ValueAsInt;  //3.0<

  Result := LoanPrin(RPayment, IntRate, TNumPay, NPayPerYr);
//  Edit5.Text := FormatFloat('#,##0.00', Result);  //3.0
  Edit5.Value := Result;                            //3.0
end;  //LPrincipal


function TFc_Form1.LTerm: integer;
var
  PrinAmt, IntRate, RPayment: Double;
  NPayPerYr: Integer;
begin
  PrinAmt := Edit1.Value;         //3.0>
  RPayment := Edit2.Value;
  IntRate := Edit3.Value;
  NPayPerYr := Edit4.ValueAsInt;  //3.0<

  Result := LoanTNP(PrinAmt, RPayment, IntRate, NPayPerYr);
  if Result = 0 then
  begin
    Edit5.Text := '0.00';
    MessageDlg('Your loan payments are too small to achieve amortization',
      mtError, [mbOK], 0);
  end
  else
    //  Edit5.Text := Format('%10.0d', [Result]);  //3.0
    Edit5.Value := Result;                         //3.0
end;  //LTerm


function TFc_Form1.AFValue: Double;
var
  DepAmt, IntRate: Double;
  TNumDep, NDepPerYr: Integer;
begin
  DepAmt := Edit1.Value;          //3.0
  IntRate := Edit2.Value;
  TNumDep := Edit3.ValueAsInt;
  NDepPerYr := Edit4.ValueAsInt;  //3.0<

  Result := AnnuFV(DepAmt, IntRate, TNumDep, NDepPerYr);
//  Edit5.Text := FormatFloat('#,##0.00', Result);  //3.0
  Edit5.Value := Result;                            //3.0
end;  //AFValue


function TFc_Form1.ADeposit: Double;
var
  IntRate, FinalVal: Double;
  TNumDep, NDepPerYr: Integer;
begin
  FinalVal := Edit1.Value;        //3.0>
  IntRate := Edit2.Value;
  TNumDep := Edit3.ValueAsInt;
  NDepPerYr := Edit4.ValueAsInt;  //3.0<

  Result := AnnuDep(FinalVal, IntRate, TNumDep, NDepPerYr);
//  Edit5.Text := FormatFloat('#,##0.00', Result);  //3.0
  Edit5.Value := Result;                            //3.0
end;  //ADeposit


function TFc_Form1.ATerm: integer;
var
  DepAmt, IntRate, FinalVal: Double;
  NDepPerYr: Integer;
begin
  DepAmt := Edit1.Value;           //3.0>
  FinalVal := Edit2.Value;
  IntRate := Edit3.Value;
  NDepPerYr := Edit4.ValueAsInt;   //3.0<

  Result := AnnuTND(DepAmt, FinalVal, IntRate, NDepPerYr);
//  Edit5.Text := Format('%10.0d', [Result]);  //3.0
  Edit5.Value := Result;                       //3.0
end;  //ATerm


function TFc_Form1.WDrawPrin: Double;
var
  WdAmt, IntRate: Double;
  TNumWd, NWdPerYr: integer;
begin
  WdAmt := Edit1.Value;          //3.0>
  IntRate := Edit2.Value;
  TNumWd := Edit3.ValueAsInt;
  NWdPerYr := Edit4.ValueAsInt;  //3.0<

  Result := WDrawP(WdAmt, IntRate, TNumWd, NWdPerYr);
//  Edit5.Text := FormatFloat('#,##0.00', Result);  //3.0
  Edit5.Value := Result;                            //3.0
end;  //WDrawPrin


function TFc_Form1.WDrawAmt: Double;
var
  WdPrin, IntRate: Double;
  TNumWd, NWdPerYr: integer;
begin
  WdPrin := Edit1.Value;         //3.0>
  IntRate := Edit2.Value;
  TNumWd := Edit3.ValueAsInt;
  NWdPerYr := Edit4.ValueAsInt;  //3.0<

  Result := WDrawWD(WdPrin, IntRate, TNumWd, NWdPerYr);
//  Edit5.Text := FormatFloat('#,##0.00', Result);   //3.0
  Edit5.Value := Result;                             //3.0
end;  //WDrawAmt


function TFc_Form1.WDTerm: integer;
var
  WdAmt, WdPrin, IntRate: Double;
  NWdPerYr: integer;
begin
  WdPrin := Edit1.Value;          //3.0>
  WdAmt := Edit2.Value;
  IntRate := Edit3.Value;
  NWdPerYr := Edit4.ValueAsInt;   //3.0<

  Result := WDrawTNW(WdPrin, WdAmt, IntRate, NWdPerYr);
  if Result = 0 then
  begin
    Edit5.Text := '0';
    MessageDlg('Your withdrawals are not large enough to deplete the annuity',
      mtError, [mbOK], 0);
  end
  else
//    Edit5.Text := IntToStr(Result);  //3.0
    Edit5.Value := Result;             //3.0
end;  //WDTerm



{ ======= Mortgage Processes ======== }

procedure TFc_Form1.Mortgage1Click(Sender: TObject);  //2.0 >>
begin
  CancelBtnClick(Self);  //2.1
  FinFunc := 14;
  InValues := False;  //2.2>>
  File1.Enabled := False;
  Investment1.Enabled := False;
  Loans1.Enabled := False;
  Annuity1.Enabled := False;  //2.2<<
  Image1.Visible := False;  //3.0
  Panel7.Visible := True;
  Edit6.Enabled := True;
  Edit7.Enabled := True;
  Edit8.Enabled := True;
  Edit9.Enabled := True;
  CalcBtn.Enabled := True;
  RptBtn.Enabled := True;
  CancelBtn.Enabled := True;
  Caption := 'Financial Calculator (Mortgage)';  //2.1
  if NewMortgData = True then
    ResetEdit629;  //3.0
  Edit6.SetFocus;
  if InValues = True then
  begin
    Panel7.Visible := False;
    ProcessMortgage;
  end;
end;  //Mortgage1Click  //2.0 <<


procedure TFc_Form1.ProcessMortgage;  //2.0 >>
var
  Principal, Payment, InterestRate, LastPayment: Double;
  Balance, Interest, IntFactor, Amortized: Double;
  TotalPayments, TotalInterest, TotalAmort: Double;
  TotalNumPay, NumPayPerYr, PayNum: integer;
  TotalNumMon, NumPayPerMon: integer;
  Header1, Header2, Header3, Header4: string;
  oneLine, st1, st2, st3, st4, st5, st6, st7, st8: string;
//  ErrMsg: string;
begin
  FinFunc := 14;
  Label1.visible := False;
  Panel1.visible := False;
  Panel2.Caption := '';
  Print1.Enabled := True;
  PrintSetup1.Enabled := True;
  Fc_Memo1.Align := alClient;
  Fc_Memo1.Clear;
  GetOut := False;
  Label1.Caption := ' MORTGAGE - Analysis of loan repayments';
  Header1 := '             MORTGAGE - Analysis of loan repayments';
  Header2 := ' ===============================================================';
  Header3 := '                       Repayment Schedule                       ';
  Header4 := ' ---------------------------------------------------------------';

  if not InValues then
    exit;
  if GetOut = True then
    Exit;

  // Convert input data
  Principal := Edit6.Value;   //3.0>
  InterestRate := Edit7.Value;
  TotalNumMon := Edit8.ValueAsInt;
  NumPayPerMon := Edit9.ValueAsInt;   //3.0<

  // Process and display data, also create Mortgage.txt file
  TotalNumPay := TotalNumMon * NumPayPerMon;
  NumPayPerYr := 12 * NumPayPerMon;
  Payment := LoanPay(Principal, InterestRate, TotalNumPay, NumPayPerYr);
  Payment := Round(Payment * 100) /100;  //2.1
  Balance := Principal;
  IntFactor := InterestRate / NumPayPerYr;
  TotalPayments := 0.0;
  TotalInterest := 0.0;
  TotalAmort := 0.0;
  PayNum := 0;

  st1 := ' Principal(amount borrowed) = ' + Format('%-10.2f', [Principal]);
  st2 := ' Annual Interest Rate(%)    = ' + Format('%-2.6f', [InterestRate]);
  st3 := ' Term of loan (in months)   = ' + Format('%-5d', [TotalNumMon]);
  st4 := ' Number payments per month  = ' + Format('%-1d', [NumPayPerMon]);
  st5 := ' Regular Payment per period = ' + Format('%-10.2f', [Payment]);

{$I-}
  AssignFile(OUTPUT, 'MORTGAGE.TXT');
  Rewrite(OUTPUT);

  writeln(OUTPUT, Header1);
  writeln(OUTPUT, Header2);
  writeln(OUTPUT, st1);
  writeln(OUTPUT, st2);
  writeln(OUTPUT, st3);
  writeln(OUTPUT, st4);
  writeln(OUTPUT, st5);
  writeln(OUTPUT, Header2);
  writeln(OUTPUT, Header3);
  writeln(OUTPUT, Header4);

  repeat
    PayNum := PayNum + 1;
    Interest := Balance * IntFactor / 100.0;
    Interest := Round(Interest * 100) / 100; //2.1
    if PayNum = TotalNumPay then
      Payment := Balance + Interest;
    LastPayment := Payment;
    Amortized := Payment - Interest;
    Balance := Balance - Amortized;
    if Balance < 0.0 then
    begin
      Payment := Payment + Balance;
      LastPayment := Payment;
      Amortized := Amortized + Balance;
      Balance := 0.0;
    end;
    TotalPayments := TotalPayments + Payment;
    TotalInterest := TotalInterest + Interest;
    TotalAmort := TotalAmort + Amortized;
    oneLine := Format('%5d %13.2f %10.2f %10.2f %10.2f %11.2f',
      [PayNum, Balance, Interest, TotalInterest, Amortized, TotalAmort]);
    writeln(OUTPUT, oneLine);
  until
    (PayNum = TotalNumPay) or (Balance = 0.0);
  writeln(OUTPUT, Header2);
  writeln;
  st6 := ' Last payment      = ' + Format('%11.2f', [LastPayment]);
  st7 := ' Total payments    = ' + Format('%11.2f', [TotalPayments]);
  st8 := ' Cost of borrowing = ' + Format('%11.2f', [(TotalPayments - Principal)]);
  writeln(OUTPUT, st6);
  writeln(OUTPUT, st7);
  writeln(OUTPUT, st8);
  System.CloseFile(OUTPUT);

  Fc_Memo1.visible := True;
  ClearBtn2.Visible := True;
  PrintBtn.Visible := True;
  Label1.visible := True;
  Label6.Caption := 'MORTGAGE - Payment per period';
  Label6.Visible := True;
  Edit5.Text := FormatFloat('#,##0.00', Payment);

{$IFDEF D4_OR_HIGHER}
  AssignFile(INPUT, 'MORTGAGE.TXT');
  Reset(INPUT);
{$ENDIF}
  Fc_Memo1.Lines.LoadFromFile('MORTGAGE.TXT');
  Fc_Memo1.Modified := False;
{$IFDEF D4_OR_HIGHER}
  System.CloseFile(INPUT);
{$ENDIF}
{$I+}
end; //ProcessMortgage  //2.0 <<


procedure TFc_Form1.PrintBtnClick(Sender: TObject);
var
  Line, i: integer;
  PrintText: System.Text;
begin
  if PrintDialog1.Execute then
  begin
    AssignPrn(PrintText);
    Rewrite(PrintText);

    Printer.Canvas.Font.Name := 'Courier New';
    Printer.Canvas.Font.Size := 10;

    writeln(PrintText, #10 + #10 + #10 + #10 + #13);

    for i := 0 to 7 do
      writeln(PrintText, '          ' +
        Fc_memo1.Lines[i]);

    writeln(PrintText, '          ' +
      ' Payment  Remaining   ---Interest Paid---   --Amount Amortized--');
    writeln(PrintText, '          ' +
      ' Number   Balance     This Time   To Date   This Time    To Date');
    Line := 9;
    while Line < Fc_Memo1.Lines.Count do
    begin
      writeln(PrintText, '          ' +
        Fc_memo1.Lines[Line]);
      Line := Line + 1;
      if Line mod 56 = 0 then
        write(PrintText, #12 + #10 + #10 + #10 + #10 + #13);
    end;
    System.Close(PrintText);
    if MessageDlg('Required data have been sent to printer.' + #13 +
                  'Do you want to close Mortgage-Calculation Form?',
                  mtConfirmation, [mbYes, mbNo], 0) = mrYes then   //2.2>>
      CancelBtnClick(Self);  //2.2<<
  end;
end;  //PrintBtnClick


{ ==========  Help Routines ========== }


procedure TFc_Form1.Contents1Click(Sender: TObject);
begin
{$IFDEF D6_OR_HIGHER}  //3.0hh
  Application.HelpFile := 'FIN_CALC.chm';     //3.0hh
  Application.HelpCommand(Help_Contents, 0);  //3.0hh
//  Application.HelpSystem.ShowTableOfContents;  //3.0hh
{$ELSE}
  Application.HelpFile := 'FIN_CALC.hlp';
  Application.HelpCommand(HELP_FINDER, 0);
{$ENDIF}
end;  //Contents1Click

procedure TFc_Form1.HelpBtn1Click(Sender: TObject);
var
  ContxtID: integer;
  Topic: String;
begin
  if (FinFunc in [1..14]) then
  begin
    ContxtID  := 0;
    case FinFunc of
      1:  begin
            ContxtID := 1011;
            Topic    := 'IDH_11-Investment';
          end;
      2:  begin
            ContxtID := 1012;
            Topic    := 'IDH_12-Investment';
          end;
      3:  begin
            ContxtID := 1013;
            Topic    := 'IDH_13-Investment';
          end;
      4:  begin
            ContxtID := 1014;
            Topic    := 'IDH_14-Investment';
          end;
      5:  begin
            ContxtID := 1021;
            Topic    := 'IDH_21-Loans';
          end;
      6:  begin
            ContxtID := 1022;
            Topic    := 'IDH_22-Loans';
          end;
      7:  begin
            ContxtID := 1023;
            Topic    := 'IDH_23-Loans';
          end;
      8:  begin
            ContxtID := 1031;
            Topic    := 'IDH_31-Annuity';
          end;
      9:  begin
            ContxtID := 1032;
            Topic    := 'IDH_32-Annuity';
          end;
      10: begin
            ContxtID := 1033;
            Topic    := 'IDH_33-Annuity';
          end;
      11: begin
            ContxtID := 1041;
            Topic    := 'IDH_41-Annuity';
          end;
      12: begin
            ContxtID := 1042;
            Topic    := 'IDH_42-Annuity';
          end;
      13: begin
            ContxtID := 1043;
            Topic    := 'IDH_43-Annuity';
          end;
      14: begin
            ContxtID := 1051;
            Topic    := 'IDH_51-Mortgage';
          end;
    end;
    Fc_Form1.HelpContext := ContxtID;
    Panel1.HelpContext := ContxtID;
    Panel5.HelpContext := ContxtID;
    Panel6.HelpContext := ContxtID;
    Panel7.HelpContext := ContxtID;
    Application.HelpContext(ContxtID);
  end
  else begin    //show contents
{$IFDEF D6_OR_HIGHER}  //3.0hh
    Application.HelpSystem.ShowTableOfContents;
//    Application.HelpCommand(HELP_CONTENTS, 0);
{$ELSE}
    Application.HelpCommand(HELP_FINDER, 0);
{$ENDIF}
  end;
end;  //HelpBtn1Click


procedure TFc_Form1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_F1) then
  begin
    HelpBtn1Click(Self);
    Key := 0;
  end;
end;   //FormKeyDown


end.





